#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h>
#include "postgres.h"
#include "fmgr.h"
#include "executor/spi.h"


char cod[8];			/* codice di otto lettere */


PG_FUNCTION_INFO_V1(genera_codice);

Datum genera_codice(PG_FUNCTION_ARGS) {

  int i;
  char n;			/* numero pseudo-casuale */
  char c[9];			/* codice di otto lettere + NULL */
  char query[1024];		/* testo della query per selezionare il codice */
  Name tablename = PG_GETARG_NAME(0);
  Name fieldname = PG_GETARG_NAME(1);

  elog(DEBUG, "tablename: %s", tablename->data);
  elog(DEBUG, "fieldname: %s", fieldname->data);

  /* Connette la procedura all'SPI manager */
  if (SPI_connect() != SPI_OK_CONNECT)
    PG_RETURN_NULL();

  do {
	/* Genera il codice di otto lettere */
	for (i = 0; i < 8; i++) {
	  srandomdev();
	  n = (char)(random() % 62);	/* Numero pseudo-casuale compreso tra 0 e 61 */
	  if (n <= 25) {
		/* 0 <= n <= 25: lettera tra 'A' e 'Z' (alfabeto inglese) */
		cod[i] = n + 'A';
		c[i] = cod[i];
	  } else if (n <= 51) {
		/* 26 <= n <= 51: lettera tra 'a' e 'z' (alfabeto inglese) */
		cod[i] = (n - 26) + 'a';
		c[i] = cod[i];
	  } else {
		/* 52 <= n <= 61: cifra tra '0' e '9' */
		cod[i] = (n - 52) + '0';
		c[i] = cod[i];
	  }
	}
	c[i] = '\0';

	elog(DEBUG, "Generated code: %s", c);

	/*
	 * Prepara il testo della query che verifica se nella base di dati
	 * e' gia' presente il codice appena generato
	 */
	bzero(query, sizeof(query));
	(void)strlcat(query, "SELECT ", sizeof(query));
	(void)strlcat(query, fieldname->data, sizeof(query));
	(void)strlcat(query, " FROM ", sizeof(query));
	(void)strlcat(query, tablename->data, sizeof(query));
	(void)strlcat(query, " WHERE ", sizeof(query));
	(void)strlcat(query, fieldname->data, sizeof(query));
	(void)strlcat(query, " = '", sizeof(query));
	(void)strlcat(query, c, sizeof(query));
	(void)strlcat(query, "';", sizeof(query));

	elog(DEBUG, "Query text: %s", query);

	/* Esegue la query per cercare il codice generato */
	if (SPI_exec(query, 1) != SPI_OK_SELECT) {
	  SPI_finish();
	  PG_RETURN_NULL();
	}

	/*
	 * Se la query ritorna 0 tuple allora il codice non e' presente nella base
	 * di dati e il ciclo puo' terminare.
	 * In caso contrario (la query ritorna una tupla) e' necessario eseguire un
	 * altro ciclo per generare un nuovo codice.
	 */
  } while (SPI_processed > 0);

  /* Disconnette la procedura dall'SPI manager */
  SPI_finish();

  PG_RETURN_BPCHAR_P(cod);
}
